@@ -6,7 +6,7 @@ django-bitfield
 try:
     VERSION = __import__('pkg_resources') \
         .get_distribution('bitfield').version
-except Exception, e:
+except Exception:
     VERSION = 'unknown'
 
-from bitfield.models import Bit, BitHandler, CompositeBitField, BitField
\ No newline at end of file
+from bitfield.models import Bit, BitHandler, CompositeBitField, BitField  # noqa
