@@ -1,7 +1,11 @@
+from django.db.models import F
+from django.core.exceptions import ValidationError
 from django.utils.translation import ugettext_lazy as _
 from django.contrib.admin import FieldListFilter
+from django.contrib.admin.options import IncorrectLookupParameters
 
 from bitfield import Bit
+from bitfield.compat import bitor
 
 
 class BitFieldListFilter(FieldListFilter):
@@ -13,8 +17,16 @@ class BitFieldListFilter(FieldListFilter):
         self.lookup_kwarg = field_path
         self.lookup_val = int(request.GET.get(self.lookup_kwarg, 0))
         self.flags = field.flags
-        super(BitFieldListFilter, self).__init__(field,
-            request, params, model, model_admin, field_path)
+        self.labels = field.labels
+        super(BitFieldListFilter, self).__init__(
+            field, request, params, model, model_admin, field_path)
+
+    def queryset(self, request, queryset):
+        filter = dict((p, bitor(F(p), v)) for p, v in self.used_parameters.iteritems())
+        try:
+            return queryset.filter(**filter)
+        except ValidationError as e:
+            raise IncorrectLookupParameters(e)
 
     def expected_parameters(self):
         return [self.lookup_kwarg]
@@ -22,16 +34,13 @@ class BitFieldListFilter(FieldListFilter):
     def choices(self, cl):
         yield {
             'selected': self.lookup_val == 0,
-            'query_string': cl.get_query_string({
-                }, [self.lookup_kwarg]),
+            'query_string': cl.get_query_string({}, [self.lookup_kwarg]),
             'display': _('All'),
         }
         for number, flag in enumerate(self.flags):
             bit_mask = Bit(number).mask
             yield {
                 'selected': self.lookup_val == bit_mask,
-                'query_string': cl.get_query_string({
-                        self.lookup_kwarg: bit_mask,
-                    }),
-                'display': flag,
+                'query_string': cl.get_query_string({self.lookup_kwarg: bit_mask}),
+                'display': self.labels[number],
             }
