@@ -8,6 +8,8 @@ except ImportError:
     # django 1.2
     from django.db.models.fields.subclassing import LegacyConnection as SubfieldBase  # NOQA
 
+import six
+
 from bitfield.forms import BitFormField
 from bitfield.query import BitQueryLookupWrapper
 from bitfield.types import BitHandler, Bit
@@ -93,9 +95,7 @@ class BitFieldMeta(SubfieldBase):
         return new_class
 
 
-class BitField(BigIntegerField):
-    __metaclass__ = BitFieldMeta
-
+class BitField(six.with_metaclass(BitFieldMeta, BigIntegerField)):
     def __init__(self, flags, default=None, *args, **kwargs):
         if isinstance(flags, dict):
             # Get only integer keys in correct range
@@ -108,6 +108,15 @@ class BitField(BigIntegerField):
         if len(flags) > MAX_FLAG_COUNT:
             raise ValueError('Too many flags')
 
+        flags = list(flags)
+        labels = []
+        for num, flag in enumerate(flags):
+            if isinstance(flag, (tuple, list)):
+                flags[num] = flag[0]
+                labels.append(flag[1])
+            else:
+                labels.append(flag)
+
         if isinstance(default, (list, tuple, set, frozenset)):
             new_value = 0
             for flag in default:
@@ -116,6 +125,7 @@ class BitField(BigIntegerField):
 
         BigIntegerField.__init__(self, default=default, *args, **kwargs)
         self.flags = flags
+        self.labels = labels
 
     def south_field_triple(self):
         "Returns a suitable description of this field for South."
@@ -125,7 +135,8 @@ class BitField(BigIntegerField):
         return (field_class, args, kwargs)
 
     def formfield(self, form_class=BitFormField, **kwargs):
-        return Field.formfield(self, form_class, choices=[(k, k) for k in self.flags], **kwargs)
+        choices = [(k, self.labels[self.flags.index(k)]) for k in self.flags]
+        return Field.formfield(self, form_class, choices=choices, **kwargs)
 
     def pre_save(self, instance, add):
         value = getattr(instance, self.attname)
@@ -147,7 +158,7 @@ class BitField(BigIntegerField):
         if isinstance(value, (BitHandler, Bit)):
             return BitQueryLookupWrapper(self.model._meta.db_table, self.db_column or self.name, value)
         return BigIntegerField.get_db_prep_lookup(self, lookup_type=lookup_type, value=value,
-                                                        connection=connection, prepared=prepared)
+                                                  connection=connection, prepared=prepared)
 
     def get_prep_lookup(self, lookup_type, value):
         if isinstance(value, SQLEvaluator) and isinstance(value.expression, Bit):
@@ -165,13 +176,13 @@ class BitField(BigIntegerField):
             # Regression for #1425: fix bad data that was created resulting
             # in negative values for flags.  Compute the value that would
             # have been visible ot the application to preserve compatibility.
-            if isinstance(value, (int, long)) and value < 0:
+            if isinstance(value, six.integer_types) and value < 0:
                 new_value = 0
                 for bit_number, _ in enumerate(self.flags):
                     new_value |= (value & (2 ** bit_number))
                 value = new_value
 
-            value = BitHandler(value, self.flags)
+            value = BitHandler(value, self.flags, self.labels)
         else:
             # Ensure flags are consistent for unpickling
             value._keys = self.flags
@@ -229,7 +240,7 @@ class CompositeBitField(object):
         cls = sender
         model_fields = dict([
             (f.name, f) for f in cls._meta.fields if f.name in self.fields])
-        all_flags = sum([model_fields[f].flags for f in self.fields], ())
+        all_flags = sum([model_fields[f].flags for f in self.fields], [])
         if len(all_flags) != len(set(all_flags)):
             raise ValueError('BitField flags must be unique.')
 
